
from gpsinterface import *
from geometry import *
from gpxutils import *
from urlutil import *
import shelve, urllib

class GeoErrandApp:

	def __init__(self):
		self.gps = None
		try:
			self.gps = GpsGypsy()
			print "Started Gypsy Interface"
		except:
			print "Cannot init Gypsy Interface"

		if self.gps is None:
			try:
				self.gps = GpsDgps()
				print "Started DGPS Interface"
			except:
				print "Cannot init DGPS Interface"

		#Connect signals
		if self.gps is not None:
			self.gps.onPositionChanged = self.onPositionChanged
			self.gps.onCourseChanged = self.onCourseChanged
			self.gps.onConnectionStatusChanged = self.onConnectionStatusChanged

		baseFolder = os.path.dirname(os.path.realpath(__file__))
		self.guiRefreshFunc = None
		self.lat = None
		self.lon = None
		self.alt = None
		self.heading = None
		self.waypoints = shelve.open(os.path.join(baseFolder,"waypoints.db"))
		self.targetErrand = None
		self.targetId = None
		
		self.annotationDb = shelve.open(os.path.join(baseFolder,"annotation.db"))

	def ClearPositionData(self):
		ke = self.waypoints.keys()
		for k in ke:
			self.waypoints[str(k)] = None
			del self.waypoints[str(k)]

	def ImportGpxData(self,filename=None):
		baseFolder = os.path.dirname(os.path.realpath(__file__))
		if filename is None: filename = os.path.join(baseFolder,"missing.gpx")
		self.ClearPositionData()
		gpxreader = GpxReader()
		gpxreader.Read(filename)
		for way,i in zip(gpxreader.waypoints,range(len(gpxreader.waypoints))):
			self.waypoints[str(i)] = way
		#print self.waypoints
		self.waypoints.sync()

    	def onCourseChanged(self, tstamp, speed, heading, climb):
		self.heading = heading		
		#if self.guiRefreshFunc is not None: self.guiRefreshFunc()

	def onPositionChanged(self, tstamp, lat, lon, alt):
		#print "position changed", tstamp
		self.lat = lat
		self.lon = lon
		self.alt = alt

		if self.guiRefreshFunc is not None: self.guiRefreshFunc()

	def onConnectionStatusChanged(self,constatus):
		#self.status.set_text("Gypsy connected "+str(constatus))

		if self.guiRefreshFunc is not None: self.guiRefreshFunc()
		
	def SetTargetErrand(self, targetErrand):
		self.targetId = targetErrand
		self.targetErrand = self.waypoints[str(targetErrand)]		

	def GetTargetName(self):
		if self.targetErrand is None: return None
		return self.targetErrand['name']
	def GetTargetPos(self):

		if self.targetErrand is None: return (None, None)
		return (self.targetErrand['lat'], self.targetErrand['lon'])
	def GetTargetId(self):
		return 1

	def SetTextAnnotation(self, text):
		#print "saving text annotation",self.targetId, str(text)
		if str(self.targetId) is None: return
		if str(self.targetId) not in self.annotationDb.keys():
			self.annotationDb[str(self.targetId)] = {}
		record = self.annotationDb[str(self.targetId)]
		record['text'] = str(text)
		self.annotationDb[str(self.targetId)] = record
		self.annotationDb.sync()

	def GetTextAnnotation(self, i = None):
		if i is None: i = self.targetId
		#print "Getting text",self.targetId
		if str(i) is None: return
		if str(i) not in self.annotationDb:
			return None
		if 'text' not in self.annotationDb[str(i)]:
			return None
		#print self.annotationDb[str(i)]['text']
		return self.annotationDb[str(i)]['text']

	def SetAnnotatePositionHere(self):
		#print "SetAnnotatePositionHere"
		if str(self.targetId) is None: return
		if str(self.targetId) not in self.annotationDb.keys():
			self.annotationDb[str(self.targetId)] = {}
		record = self.annotationDb[str(self.targetId)]
		record['position'] = (float(self.lat),float(self.lon))
		self.annotationDb[str(self.targetId)] = record
		self.annotationDb.sync()

	def ClearAnnotatePosition(self):
		#print "ClearAnnotatePosition"
		if str(self.targetId) is None: return
		if str(self.targetId) not in self.annotationDb.keys():
			self.annotationDb[str(self.targetId)] = {}
		record = self.annotationDb[str(self.targetId)]
		record['position'] = None
		self.annotationDb[str(self.targetId)] = record
		self.annotationDb.sync()
		
	def GetAnnotatePosition(self, i = None):
		if i is None: i = self.targetId
		if str(i) is None: return None
		if str(i) not in self.annotationDb.keys():
			self.annotationDb[str(i)] = {}
		record = self.annotationDb[str(i)]
		if 'position' not in record: return None
		return record['position']

	def UploadToInternet(self):
		print "Uploading Annotations"

		out = "<annotations>"
		for k in self.annotationDb.keys():
			#print "k=",type(k)
			#print "k is None",(k == "None")
			if k == None: continue
			if k == "None": continue
			record = self.annotationDb[k]
			textAnnot = "text" in record and len(record['text']) > 0
			if not textAnnot and "position" not in record:
				continue
			#print "k2=",type(k)
			way = self.waypoints[k]
			out = out + "<annotation lat='"+str(way['lat'])+"' lon='"+str(way['lon'])+"'"
			if way['externalId'] is not None:
				out = out + " id='"+way['externalId']+"'"
			out = out + ">"
			if "text" in record and len(record['text']) > 0:
				out = out + "<text>" + record['text'] + "</text>"
			if "position" in record:
				out = out+"<position lat='"+str(record['position'][0])+"' lon='"+str(record['position'][1])+"'>"

			out = out + "</annotation>"
		out = out + "</annotations>"

		fi=open("annotationsOut.xml","wt")
		fi.write(out)
		fi.flush()

		print out

		postFields = {"annotation":out}
		postEnc = urllib.urlencode(postFields)
		(bret,hret) = Post("http://timsc.dev.openstreetmap.org/geoerrand/upload/rxpost.php",postEnc)		
		print hret
		print bret

	def DownloadErrands(self):
		print "Downloading Errands"
		(bret,hret) = Get("http://timsc.dev.openstreetmap.org/geoerrand/missing.gpx")		
		print hret
		print bret
		baseFolder = os.path.dirname(os.path.realpath(__file__))
		fi = open(os.path.join(baseFolder,"missing.gpx"),"wt")
		fi.write(bret)
		fi.flush()
		self.ClearAnnotations()
		self.ImportGpxData()

	def LoadErrands(self, filename):
		print "Loading Errands from",filename
		baseFolder = os.path.dirname(os.path.realpath(__file__))
		self.ClearAnnotations()
		self.ImportGpxData(os.path.join(baseFolder,"missing.gpx"))

	def ClearAnnotations(self):
		print "Clearing Annotations"
		ke = self.annotationDb.keys()
		for k in ke:
			self.annotationDb[str(k)] = {}
			del self.annotationDb[str(k)]



