
import math

def LatLonToCartesian(lat, lon):
	#Assume spherical Earth
	r = 6378.1 * 1000. # in metres

	lat = math.radians(lat)
	lon = math.radians(lon)

	x = r * math.sin(lat) * math.cos(lon)
	y = r * math.sin(lat) * math.sin(lon)
	z = r * math.cos(lat)

	return (x,y,z)

def PythagDistCart(p1, p2):
	out = 0.
	for i in range(len(p1)):
		out += pow(p1[i] - p2[i],2.0)
	return pow(out,0.5)

def PythagDistLatLon(p1, p2):
	p1c = LatLonToCartesian(*p1)
	p2c = LatLonToCartesian(*p2)
	return PythagDistCart(p1c, p2c)

def VecMagnitude(vec):
	out = 0.
	for c in vec:
		out = out + pow(c,2.)
	return pow(out,0.5)

def NormaliseVec(vec):
	mag = VecMagnitude(vec)
	if mag == 0.: return vec #Zero length
	out = []
	for c in vec:
		out.append(c / mag)
	return out

def SubtractVec(v1,v2):
	out = []
	for i in range(len(v1)):
		out.append(v1[i] - v2[i])
	return out

def ScaleVec(v, scale):
	out = []
	for c in v:
		out.append(c * scale)
	return out

def VecDotProduct(v1, v2):
	out = 0.
	for i in range(len(v1)):
		out = out + v1[i] * v2[i]
	return out

def DifferenceInBearing(myBearing, targetBearing):
	solutions = []
	solutions.append(targetBearing - myBearing)
	solutions.append(360. + targetBearing - myBearing)
	solutions.append(targetBearing - 360. - myBearing)
	return min(solutions)

def GetBearing(pfrom, pto):
	cpfrom = LatLonToCartesian(*pfrom)
	cpto = LatLonToCartesian(*pto)
	direction = NormaliseVec(SubtractVec(cpto, cpfrom))

	#Get vector towards equator
	if pfrom[0] > 0.: equator = (pfrom[0]-0.01, pfrom[1]) #Northern hemisphere
	else: equator = (pfrom[0]+0.01, pfrom[1]) #Southern

	cequator = LatLonToCartesian(*equator)
	fromToEqu = SubtractVec(cequator, cpfrom)
	
	if pfrom[0] > 0.: northVec = ScaleVec(fromToEqu,-1.) #Flip vector around if in north
	else: northVec = fromToEqu
	northVec = NormaliseVec(northVec)
	
	#Get east vector
	eastpos = LatLonToCartesian(pfrom[0], pfrom[1]+0.01)
	eastVec = SubtractVec(eastpos, cpfrom)
	eastVec = NormaliseVec(eastVec)

	#Get dot products
	ncomp = VecDotProduct(direction, northVec)
	ecomp = VecDotProduct(direction, eastVec)
	bearing = math.atan2(ecomp, ncomp)
	return math.degrees(bearing)

