#!/usr/bin/env python
import gobject

class GpsGypsy:
	def __init__(self):
		import dbus

		name = 'org.freesmartphone.ogpsd'
		path = '/org/freedesktop/Gypsy'

		bus = dbus.SystemBus()
		usageObj = bus.get_object('org.freesmartphone.ousaged', '/org/freesmartphone/Usage')
		dbus.Interface(usageObj, 'org.freesmartphone.Usage').RequestResource('GPS')
		self.useobj = bus.get_object(name, path)

		bus.add_signal_receiver(self.PosChanged, 'PositionChanged', 'org.freedesktop.Gypsy.Position', name, path)
		bus.add_signal_receiver(self.ConnectionChanges, 
			'ConnectionStatusChanged', 'org.freedesktop.Gypsy.Device', name, path)
		bus.add_signal_receiver(self.CourseChanged, 'CourseChanged', 'org.freedesktop.Gypsy.Course', name, path)

		self.onConnectionStatusChanged = None
		self.onPositionChanged = None
		self.onCourseChanged = None

		gobject.timeout_add(1000, self.Update)

	def Update(self):
		#print "Gypsy: Timer func"
		return True

	def ConnectionChanges(self,constatus):
		#self.status.set_text("Gypsy connected "+str(constatus))
		if self.onConnectionStatusChanged is not None:
			self.onConnectionStatusChanged(constatus)

	def PosChanged(self,fields, tstamp, lat, lon, alt):
		#print "Gypsy: Position changed"
		latValid = (fields & (1 << 0)) > 0
		lonValid = (fields & (1 << 1)) > 0
		altValid = (fields & (1 << 2)) > 0

		if not latValid: lat = None
		if not lonValid: lon = None
		if not altValid: alt = None

		if self.onPositionChanged is not None:
			self.onPositionChanged(tstamp, lat, lon, alt)

	def CourseChanged(self, fields, tstamp, speed, heading, climb):

		speedValid = (fields & (1 << 0)) > 0
		courseValid = (fields & (1 << 1)) > 0
		climbValid = (fields & (1 << 2)) > 0

		if not speedValid: speed = None
		if not courseValid: heading = None
		if not climbValid: climb = None
	
		if self.onCourseChanged is not None:
			self.onCourseChanged(tstamp, speed, heading, climb)

class GpsDgps:

	def __init__(self):
		import gps
		self.onConnectionStatusChanged = None
		self.onPositionChanged = None
		self.onCourseChanged = None

		self.g = gps.gps(mode=gps.WATCH_NEWSTYLE)
		gobject.timeout_add(1000, self.Update)

		#Keep a local copy of this figures to check if they change
		self.mode = 1 #No fix

		self.latitude = None
		self.longitude = None
		self.altitude = None

		self.speed = None
		self.track = None 
		self.climb = None

	def Update(self):
		import gps
		try:
			self.g.poll()
			if gps.PACKET_SET:
				self.g.stream()
		except:
			print "Updating python gps interface failed"

		#Check if mode changes
		if self.mode is not self.g.fix.mode:
			self.mode = self.g.fix.mode
			if self.onConnectionStatusChanged is not None:
				self.onConnectionStatusChanged(self.g.fix.mode)

		if self.g.fix.mode is gps.MODE_NO_FIX:
			return True
		if self.g.fix.mode is gps.MODE_2D:
			self.g.fix.altitude = None #Not sure this is really necessary?

		#Check if position has changed
		if self.g.fix.latitude is not self.latitude \
			or self.g.fix.longitude is not self.longitude \
			or self.g.fix.altitude is not self.altitude: posChanged = True
		else: posChanged = False

		#Update application if postion changes
		print "posChanged",posChanged,self.g.fix.latitude,self.g.fix.longitude
		if self.onPositionChanged is not None and posChanged:
			self.onPositionChanged(self.g.fix.time, 
				self.g.fix.latitude, 
				self.g.fix.longitude, 
				self.g.fix.altitude)

		self.latitude = self.g.fix.latitude
		self.longitude = self.g.fix.longitude
		self.altitude = self.g.fix.altitude

		#Check if course has changed
		if self.g.fix.speed is not self.speed \
			or self.g.fix.track is not self.track \
			or self.g.fix.climb is not self.climb: courseChanged = True
		else: courseChanged = False
		
		#Update application if course changes
		if self.onCourseChanged is not None and courseChanged:
			self.onCourseChanged(self.g.fix.time, 
				self.g.fix.speed, 
				self.g.fix.track, 
				self.g.fix.climb)

		self.speed = self.g.fix.speed
		self.track = self.g.fix.track
		self.climb = self.g.fix.climb

		return True

def ExampleConnectionStatusChanged(constatus):
	print "Connection Status",constatus

def ExamplePositionChanged(tstamp, lat, lon, alt):
	print "Position",lat,lon,alt,tstamp

def ExampleCourseChanged(tstamp, speed, heading, climb):
	print "Course",speed,heading,climb,tstamp

if __name__ == "__main__":
	from dbus.mainloop.glib import DBusGMainLoop

	DBusGMainLoop(set_as_default=True)
	mainloop = gobject.MainLoop()

	gps = None
	try:
		gps = GpsGypsy()
	except:
		print "Cannot init gypsy"

	if gps is None:
		gps = GpsDgps()
	#Connect signals
	gps.onPositionChanged = ExamplePositionChanged
	gps.onCourseChanged = ExampleCourseChanged
	gps.onConnectionStatusChanged = ExampleConnectionStatusChanged

	mainloop.run()

